@@ -88,7 +88,9 @@ class Swab(object):
                 return swabapp(environ, start_response)
 
             environ['swab.swab'] = self
-            initswabid = getswabid(environ)
+            environ['swab.id'] = initswabid = getswabid(environ)
+            if initswabid is None:
+                environ['swab.id'] = generate_id()
 
             def my_start_response(status, headers, exc_info=None):
                 swabid = getswabid(environ)
@@ -354,9 +356,10 @@ def show_variant(experiment, environ, record=True):
     variant was shown. (No deduping is done - the log line is always written. A
     page with ``show_variant`` might record multiple hits on reloads etc)
     """
-    swabid = getswabid(environ)
     swab = environ['swab.swab']
     variants = swab.experiments[experiment].variants
+    swabid = getswabid(environ)
+
     if not swab.include(experiment, environ):
         return variants[0]
 
